"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ElmWorkspace = void 0;
const fs_1 = __importDefault(require("fs"));
const globby_1 = __importDefault(require("globby"));
const os_1 = __importDefault(require("os"));
const tsyringe_1 = require("tsyringe");
const util_1 = __importDefault(require("util"));
const vscode_uri_1 = require("vscode-uri");
const elmPackageCache_1 = require("./elmPackageCache");
const forest_1 = require("./forest");
const utils = __importStar(require("./util/elmUtils"));
const path = __importStar(require("./util/path"));
const path_1 = require("./util/path");
const possibleImportsCache_1 = require("./util/possibleImportsCache");
const typeCache_1 = require("./util/types/typeCache");
const typeChecker_1 = require("./util/types/typeChecker");
const chokidar_1 = __importDefault(require("chokidar"));
const commandManager_1 = require("./commandManager");
const readFile = util_1.default.promisify(fs_1.default.readFile);
class ElmWorkspace {
    constructor(rootPath, programHost) {
        this.rootPath = rootPath;
        this.dirty = true;
        this.resolvedPackageCache = new Map();
        this.filesWatching = new Set();
        this.settings = tsyringe_1.container.resolve("Settings");
        this.connection = tsyringe_1.container.resolve("Connection");
        this.parser = tsyringe_1.container.resolve("Parser");
        this.connection.console.info(`Starting language server for folder: ${this.rootPath.toString()}`);
        this.typeCache = new typeCache_1.TypeCache();
        this.possibleImportsCache = new possibleImportsCache_1.PossibleImportsCache();
        this.operatorsCache = new Map();
        this.diagnosticsCache = new Map();
        this.host = programHost !== null && programHost !== void 0 ? programHost : this.createProgramHost();
    }
    init(progressCallback) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.initWorkspace(progressCallback);
        });
    }
    hasDocument(uri) {
        return !!this.forest.getTree(uri.toString());
    }
    isInSourceDirectory(uri) {
        return !!this.getSourceDirectoryOfFile(uri);
    }
    getSourceDirectoryOfFile(uri) {
        uri = path_1.normalizeUri(uri);
        return [
            ...this.rootProject.sourceDirectories,
            ...this.rootProject.testDirectories,
        ].find((elmFolder) => uri.startsWith(elmFolder));
    }
    getSourceFile(uri) {
        return this.getForest().getByUri(uri);
    }
    getSourceFileOfImportableModule(sourceFile, importableModuleName) {
        let moduleUri = sourceFile.project.moduleToUriMap.get(importableModuleName);
        if (!moduleUri && sourceFile.isTestFile) {
            moduleUri = sourceFile.project.testModuleToUriMap.get(importableModuleName);
        }
        if (moduleUri) {
            return this.getSourceFile(moduleUri);
        }
    }
    getForest(synchronize = true) {
        if (this.dirty && synchronize) {
            this.forest.synchronize();
            this.dirty = false;
        }
        return this.forest;
    }
    getRootPath() {
        return this.rootPath;
    }
    getTypeCache() {
        return this.typeCache;
    }
    getTypeChecker() {
        var _a;
        if (this.dirty) {
            this.forest.synchronize();
            this.dirty = false;
        }
        return (_a = this.typeChecker) !== null && _a !== void 0 ? _a : (this.typeChecker = typeChecker_1.createTypeChecker(this));
    }
    markAsDirty() {
        if (!this.dirty) {
            this.dirty = true;
            this.typeChecker = undefined;
            this.diagnosticsCache.clear();
        }
    }
    getPossibleImportsCache() {
        return this.possibleImportsCache;
    }
    getOperatorsCache() {
        return this.operatorsCache;
    }
    getSemanticDiagnostics(sourceFile, cancellationToken) {
        const cached = this.diagnosticsCache.get(sourceFile.uri);
        if (cached) {
            return cached;
        }
        const diagnostics = this.getTypeChecker().getDiagnostics(sourceFile, cancellationToken);
        this.diagnosticsCache.set(sourceFile.uri, diagnostics);
        return diagnostics;
    }
    getSemanticDiagnosticsAsync(sourceFile, cancellationToken) {
        return __awaiter(this, void 0, void 0, function* () {
            const cached = this.diagnosticsCache.get(sourceFile.uri);
            if (cached) {
                return Promise.resolve(cached);
            }
            const diagnostics = yield this.getTypeChecker().getDiagnosticsAsync(sourceFile, cancellationToken);
            this.diagnosticsCache.set(sourceFile.uri, diagnostics);
            return diagnostics;
        });
    }
    getSyntacticDiagnostics(sourceFile) {
        // Getting the type checker will bind the file if its not
        this.getTypeChecker();
        return sourceFile.parseDiagnostics;
    }
    getSuggestionDiagnostics(sourceFile, cancellationToken) {
        return this.getTypeChecker().getSuggestionDiagnostics(sourceFile, cancellationToken);
    }
    getImportableModules(sourceFile) {
        return Array.from(sourceFile.project.moduleToUriMap.entries()).map(([moduleName, uri]) => ({
            moduleName,
            uri,
        }));
    }
    initWorkspace(progressCallback) {
        return __awaiter(this, void 0, void 0, function* () {
            const clientSettings = yield this.settings.getClientSettings();
            let progress = 0;
            let elmVersion;
            try {
                elmVersion = utils.getElmVersion(clientSettings, this.rootPath, this.connection);
            }
            catch (error) {
                this.connection.console.warn(`Could not figure out elm version, this will impact how good the server works. \n ${error.stack}`);
            }
            const pathToElmJson = path.join(this.rootPath.fsPath, "elm.json");
            this.connection.console.info(`Reading elm.json from ${pathToElmJson}`);
            if (!this.filesWatching.has(pathToElmJson)) {
                this.host.watchFile(pathToElmJson, () => {
                    void this.connection.window
                        .createWorkDoneProgress()
                        .then((progress) => {
                        progress.begin("Restarting Elm Language Server", 0);
                        this.initWorkspace((percent) => {
                            progress.report(percent, `${percent.toFixed(0)}%`);
                        })
                            .then(() => progress.done())
                            .catch(() => {
                            //
                        });
                    });
                });
                this.filesWatching.add(pathToElmJson);
            }
            try {
                const elmHome = this.findElmHome();
                elmPackageCache_1.ElmPackageCache.packagesRoot = `${elmHome}/${elmVersion}/${this.packageOrPackagesFolder(elmVersion)}/`;
                // Run `elm make` to download dependencies
                try {
                    utils.execCmdSync(clientSettings.elmPath, "elm", { cmdArguments: ["make"] }, this.rootPath.fsPath, this.connection);
                }
                catch (error) {
                    // On application projects, this will give a NO INPUT error message, but will still download the dependencies
                }
                this.elmPackageCache = new elmPackageCache_1.ElmPackageCache(this.loadElmJson.bind(this));
                this.rootProject = yield this.loadRootProject(pathToElmJson);
                this.forest = new forest_1.Forest(this.rootProject);
                const elmFilePaths = yield this.findElmFilesInProject(this.rootProject);
                this.connection.console.info(`Found ${elmFilePaths.length.toString()} files to add to the project`);
                if (elmFilePaths.every((a) => a.project !== this.rootProject)) {
                    this.connection.window.showErrorMessage("The path or paths you entered in the 'source-directories' field of your 'elm.json' does not contain any elm files.");
                }
                const promiseList = [];
                const PARSE_STAGES = 3;
                const progressDelta = 100 / (elmFilePaths.length * PARSE_STAGES);
                for (const filePath of elmFilePaths) {
                    progressCallback((progress += progressDelta));
                    promiseList.push(this.readAndAddToForest(filePath, () => {
                        progressCallback((progress += progressDelta));
                    }));
                }
                yield Promise.all(promiseList);
                this.findExposedModulesOfDependencies(this.rootProject);
                commandManager_1.CommandManager.initHandlers(this.connection);
                this.connection.console.info(`Done parsing all files for ${pathToElmJson}`);
            }
            catch (error) {
                this.connection.console.error(`Error parsing files for ${pathToElmJson}:\n${error.stack}`);
            }
        });
    }
    loadRootProject(elmJsonPath) {
        return __awaiter(this, void 0, void 0, function* () {
            const elmJson = yield this.loadElmJson(elmJsonPath);
            if (elmJson.type === "application") {
                const allDependencies = new Map(Object.entries(Object.assign(Object.assign(Object.assign(Object.assign({}, elmJson.dependencies.direct), elmJson.dependencies.indirect), elmJson["test-dependencies"].direct), elmJson["test-dependencies"].indirect)).map(([dep, version]) => [dep, utils.parseVersion(version)]));
                return {
                    type: "application",
                    uri: this.rootPath.toString(),
                    sourceDirectories: elmJson["source-directories"].map((folder) => path.resolve(this.rootPath.fsPath, folder)),
                    testDirectories: [path.join(this.rootPath.fsPath, "tests")],
                    dependencies: yield this.loadDependencyMap(elmJson.dependencies.direct, allDependencies),
                    testDependencies: yield this.loadDependencyMap(elmJson["test-dependencies"].direct, allDependencies),
                    moduleToUriMap: new Map(),
                    testModuleToUriMap: new Map(),
                };
            }
            else {
                const deps = new Map(Object.entries(Object.assign(elmJson.dependencies, elmJson["test-dependencies"])).map(([dep, version]) => [dep, utils.parseConstraint(version)]));
                const solvedVersions = yield utils.solveDependencies(this.elmPackageCache, deps);
                if (!solvedVersions) {
                    this.connection.window.showErrorMessage("There is a problem with elm.json. Could not solve dependencies with the given constraints. Try running `elm make` to install missing dependencies.");
                    throw new Error("Unsolvable package constraints");
                }
                return {
                    type: "package",
                    uri: this.rootPath.toString(),
                    sourceDirectories: [path.join(this.rootPath.fsPath, "src")],
                    testDirectories: [path.join(this.rootPath.fsPath, "tests")],
                    dependencies: yield this.loadDependencyMap(elmJson.dependencies, solvedVersions),
                    testDependencies: yield this.loadDependencyMap(elmJson["test-dependencies"], solvedVersions),
                    exposedModules: new Set(utils.flattenExposedModules(elmJson["exposed-modules"])),
                    moduleToUriMap: new Map(),
                    maintainerAndPackageName: elmJson.name,
                    testModuleToUriMap: new Map(),
                };
            }
        });
    }
    loadPackage(packageName, packageVersions) {
        return __awaiter(this, void 0, void 0, function* () {
            const version = packageVersions.get(packageName);
            if (!version) {
                throw new Error("Problem getting package version");
            }
            // Version shouldn't be necessary, but it won't hurt
            const cacheKey = `${packageName}@${version.string}`;
            const cached = this.resolvedPackageCache.get(cacheKey);
            if (cached) {
                return Promise.resolve(cached);
            }
            const maintainer = packageName.substring(0, packageName.indexOf("/"));
            const name = packageName.substring(packageName.indexOf("/") + 1, packageName.length);
            const pathToPackageWithVersion = `${elmPackageCache_1.ElmPackageCache.packagesRoot}${maintainer}/${name}/${version.string}`;
            const elmJsonPath = path.join(pathToPackageWithVersion, "elm.json");
            const elmJson = yield this.loadElmJson(elmJsonPath);
            if (elmJson.type === "package") {
                const resolvedPackage = {
                    type: "package",
                    uri: vscode_uri_1.URI.file(pathToPackageWithVersion).toString(),
                    sourceDirectories: [path.join(pathToPackageWithVersion, "src")],
                    testDirectories: [path.join(pathToPackageWithVersion, "tests")],
                    dependencies: yield this.loadDependencyMap(elmJson.dependencies, packageVersions),
                    testDependencies: new Map(),
                    exposedModules: new Set(utils.flattenExposedModules(elmJson["exposed-modules"])),
                    moduleToUriMap: new Map(),
                    maintainerAndPackageName: elmJson.name,
                    testModuleToUriMap: new Map(),
                };
                this.resolvedPackageCache.set(cacheKey, resolvedPackage);
                return resolvedPackage;
            }
            else {
                throw new Error("Should never happen");
            }
        });
    }
    loadDependencyMap(deps, packageVersions) {
        return __awaiter(this, void 0, void 0, function* () {
            const dependencyMap = new Map();
            for (const dep in deps) {
                dependencyMap.set(dep, yield this.loadPackage(dep, packageVersions));
            }
            return dependencyMap;
        });
    }
    /**
     * Get all unique source directories from project dependency tree
     */
    getSourceDirectories(project) {
        const sourceDirs = new Map([
            ...project.sourceDirectories,
            ...(project === this.rootProject ? project.testDirectories : []),
        ].map((sourceDir) => [path_1.normalizeUri(sourceDir), project]));
        project.dependencies.forEach((dep) => this.getSourceDirectories(dep).forEach((project, sourceDir) => sourceDirs.set(sourceDir, project)));
        if (project === this.rootProject) {
            project.testDependencies.forEach((dep) => this.getSourceDirectories(dep).forEach((project, sourceDir) => sourceDirs.set(sourceDir, project)));
        }
        return sourceDirs;
    }
    findElmFilesInProject(project) {
        return __awaiter(this, void 0, void 0, function* () {
            const elmFilePathPromises = [];
            this.getSourceDirectories(project).forEach((project, sourceDir) => {
                elmFilePathPromises.push(this.findElmFilesInProjectWorker(sourceDir, project));
            });
            return (yield Promise.all(elmFilePathPromises)).reduce((a, b) => a.concat(b), []);
        });
    }
    findElmFilesInProjectWorker(sourceDir, project) {
        return __awaiter(this, void 0, void 0, function* () {
            const elmFiles = [];
            const maintainerAndPackageName = project.type === "package" ? project.maintainerAndPackageName : undefined;
            this.connection.console.info(`Glob ${sourceDir}/**/*.elm`);
            (yield this.host.readDirectory(sourceDir)).forEach((matchingPath) => {
                matchingPath = path_1.normalizeUri(matchingPath);
                const moduleName = utils.getModuleName(matchingPath, sourceDir);
                // We could track this using the separate `testDirectories`
                // But those are just hardcoded to ./tests anyways, so this should be fine
                const isTestDir = sourceDir.endsWith("tests");
                if (isTestDir) {
                    project.testModuleToUriMap.set(moduleName, vscode_uri_1.URI.file(matchingPath).toString());
                }
                else {
                    project.moduleToUriMap.set(moduleName, vscode_uri_1.URI.file(matchingPath).toString());
                }
                elmFiles.push({
                    maintainerAndPackageName,
                    path: matchingPath,
                    project,
                    isTestFile: isTestDir,
                });
            });
            return elmFiles;
        });
    }
    packageOrPackagesFolder(elmVersion) {
        return elmVersion === "0.19.0" ? "package" : "packages";
    }
    findElmHome() {
        const elmHomeVar = process.env.ELM_HOME;
        if (elmHomeVar) {
            return elmHomeVar;
        }
        return utils.isWindows
            ? `${os_1.default.homedir()}/AppData/Roaming/elm`
            : `${os_1.default.homedir()}/.elm`;
    }
    readAndAddToForest(filePath, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this.connection.console.info(`Adding ${filePath.path.toString()}`);
                const fileContent = yield this.host.readFile(filePath.path.toString());
                const tree = this.parser.parse(fileContent);
                this.forest.setTree(vscode_uri_1.URI.file(filePath.path).toString(), filePath.project === this.rootProject, true, tree, filePath.isTestFile, filePath.project, filePath.maintainerAndPackageName);
                callback();
            }
            catch (error) {
                this.connection.console.error(error.stack);
            }
        });
    }
    findExposedModulesOfDependencies(project) {
        // For each dependency, find every exposed module
        project.dependencies.forEach((dep) => {
            dep.moduleToUriMap.forEach((uri, module) => {
                if (dep.exposedModules.has(module)) {
                    project.moduleToUriMap.set(module, uri);
                }
            });
            this.findExposedModulesOfDependencies(dep);
        });
        if (project === this.rootProject) {
            project.testDependencies.forEach((dep) => {
                dep.moduleToUriMap.forEach((uri, module) => {
                    if (dep.exposedModules.has(module)) {
                        project.testModuleToUriMap.set(module, uri);
                    }
                });
            });
        }
    }
    loadElmJson(elmJsonPath) {
        return __awaiter(this, void 0, void 0, function* () {
            return JSON.parse(yield this.host.readFile(elmJsonPath));
        });
    }
    createProgramHost() {
        return {
            readFile: (uri) => readFile(uri, {
                encoding: "utf-8",
            }),
            readDirectory: (uri) => 
            // Cleanup the path on windows, as globby does not like backslashes
            globby_1.default(`${uri.replace(/\\/g, "/")}/**/*.elm`, {
                suppressErrors: true,
            }),
            watchFile: (uri, callback) => {
                chokidar_1.default.watch(uri).on("change", callback);
            },
        };
    }
}
exports.ElmWorkspace = ElmWorkspace;
//# sourceMappingURL=elmWorkspace.js.map