import { SyntaxNode, Tree } from "web-tree-sitter";
import { IExposing } from "./util/treeUtils";
import { SyntaxNodeMap } from "./util/types/syntaxNodeMap";
import { SymbolMap } from "./util/types/binder";
import { Diagnostic } from "./util/types/diagnostics";
import { ElmProject } from "./elmWorkspace";
export interface ITreeContainer {
    uri: string;
    writeable: boolean;
    referenced: boolean;
    maintainerAndPackageName?: string;
    tree: Tree;
    project: ElmProject;
    isTestFile: boolean;
    parseDiagnostics: Diagnostic[];
    moduleName?: string;
    resolvedModules?: Map<string, string>;
    exposing?: IExposing;
    symbolLinks?: SyntaxNodeMap<SyntaxNode, SymbolMap>;
    nonShadowableNames?: Set<string>;
}
export interface IForest {
    treeMap: Map<string, ITreeContainer>;
    getTree(uri: string): Tree | undefined;
    getByUri(uri: string): ITreeContainer | undefined;
    setTree(uri: string, writeable: boolean, referenced: boolean, tree: Tree, isTestFile: boolean, project?: ElmProject, packageName?: string): ITreeContainer;
    removeTree(uri: string): void;
    synchronize(): void;
    invalidateResolvedModules(): void;
}
export declare class Forest implements IForest {
    private rootProject;
    treeMap: Map<string, ITreeContainer>;
    constructor(rootProject: ElmProject);
    getTree(uri: string): Tree | undefined;
    getByUri(uri: string): ITreeContainer | undefined;
    setTree(uri: string, writeable: boolean, referenced: boolean, tree: Tree, isTestFile: boolean, project?: ElmProject, maintainerAndPackageName?: string): ITreeContainer;
    removeTree(uri: string): void;
    synchronize(): void;
    invalidateResolvedModules(): void;
    private resolveModules;
    private getModuleMap;
}
