"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Forest = void 0;
const imports_1 = require("./imports");
const treeUtils_1 = require("./util/treeUtils");
class Forest {
    constructor(rootProject) {
        this.rootProject = rootProject;
        this.treeMap = new Map();
    }
    getTree(uri) {
        var _a;
        return (_a = this.getByUri(uri)) === null || _a === void 0 ? void 0 : _a.tree;
    }
    getByUri(uri) {
        return this.treeMap.get(uri);
    }
    setTree(uri, writeable, referenced, tree, isTestFile, project = this.rootProject, maintainerAndPackageName) {
        tree.uri = uri;
        const treeContainer = {
            maintainerAndPackageName,
            referenced,
            tree,
            uri,
            writeable,
            project,
            isTestFile,
            parseDiagnostics: [],
        };
        this.treeMap.set(uri, treeContainer);
        return treeContainer;
    }
    removeTree(uri) {
        this.treeMap.delete(uri);
    }
    synchronize() {
        this.treeMap.forEach((sourceFile) => {
            var _a;
            // Resolve import modules
            if (!sourceFile.resolvedModules) {
                sourceFile.resolvedModules = this.resolveModules(sourceFile);
            }
            if (!sourceFile.moduleName) {
                const moduleName = (_a = treeUtils_1.TreeUtils.getModuleNameNode(sourceFile.tree)) === null || _a === void 0 ? void 0 : _a.text;
                if (moduleName) {
                    sourceFile.moduleName = moduleName;
                    if (sourceFile.project === this.rootProject &&
                        !this.getModuleMap(sourceFile).has(moduleName)) {
                        this.getModuleMap(sourceFile).set(moduleName, sourceFile.uri);
                    }
                }
            }
        });
    }
    invalidateResolvedModules() {
        this.treeMap.forEach((treeContainer) => {
            treeContainer.resolvedModules = undefined;
        });
    }
    resolveModules(sourceFile) {
        var _a;
        const importClauses = [
            ...imports_1.Imports.getVirtualImports(),
            ...((_a = treeUtils_1.TreeUtils.findAllImportClauseNodes(sourceFile.tree)) !== null && _a !== void 0 ? _a : []),
        ];
        const resolvedModules = new Map();
        importClauses.forEach((importClause) => {
            var _a;
            const moduleName = (_a = importClause.childForFieldName("moduleName")) === null || _a === void 0 ? void 0 : _a.text;
            if (moduleName) {
                let found = sourceFile.project.moduleToUriMap.get(moduleName);
                if (!found && sourceFile.isTestFile) {
                    found = sourceFile.project.testModuleToUriMap.get(moduleName);
                }
                if (found) {
                    resolvedModules.set(moduleName, found);
                }
            }
        });
        return resolvedModules;
    }
    getModuleMap(sourceFile) {
        return sourceFile.isTestFile
            ? sourceFile.project.testModuleToUriMap
            : sourceFile.project.moduleToUriMap;
    }
}
exports.Forest = Forest;
//# sourceMappingURL=forest.js.map