"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Imports = exports.resetImportsTime = exports.importsTime = void 0;
const treeUtils_1 = require("./util/treeUtils");
const tsyringe_1 = require("tsyringe");
const multiMap_1 = require("./util/multiMap");
const perf_hooks_1 = require("perf_hooks");
exports.importsTime = 0;
function resetImportsTime() {
    exports.importsTime = 0;
}
exports.resetImportsTime = resetImportsTime;
/**
 * Imports class that extends a map to handle multiple named imports
 */
class Imports extends multiMap_1.MultiMap {
    get(key, filter) {
        return super.get(key, filter, (a) => (a.explicitlyExposed ? -1 : 1));
    }
    static getImports(treeContainer, forest) {
        var _a;
        const start = perf_hooks_1.performance.now();
        const result = new Imports();
        const importNodes = [
            ...Imports.getVirtualImports(),
            ...((_a = treeUtils_1.TreeUtils.findAllImportClauseNodes(treeContainer.tree)) !== null && _a !== void 0 ? _a : []),
        ];
        importNodes.forEach((importNode) => {
            var _a, _b;
            const moduleName = (_a = importNode.childForFieldName("moduleName")) === null || _a === void 0 ? void 0 : _a.text;
            if (moduleName) {
                const uri = (_b = treeContainer.resolvedModules) === null || _b === void 0 ? void 0 : _b.get(moduleName);
                if (!uri) {
                    return;
                }
                const foundModule = forest.getByUri(uri);
                if (foundModule) {
                    const foundModuleNode = treeUtils_1.TreeUtils.findModuleDeclaration(foundModule.tree);
                    if (foundModuleNode) {
                        result.set(moduleName, {
                            alias: moduleName,
                            fromModuleName: moduleName,
                            fromUri: uri,
                            maintainerAndPackageName: foundModule.maintainerAndPackageName,
                            node: foundModuleNode,
                            type: "Module",
                            explicitlyExposed: false,
                        });
                        const exposedFromRemoteModule = foundModule.exposing;
                        if (exposedFromRemoteModule) {
                            this.getPrefixedImports(moduleName, importNode, exposedFromRemoteModule, foundModule.uri, foundModule.maintainerAndPackageName).forEach((imp) => result.set(imp.alias, imp));
                            const exposingList = importNode.childForFieldName("exposing");
                            if (exposingList) {
                                const doubleDot = exposingList.childForFieldName("doubleDot");
                                if (doubleDot) {
                                    this.getAllExposedCompletions(exposedFromRemoteModule, moduleName, foundModule.uri, foundModule.maintainerAndPackageName).forEach((imp) => result.set(imp.alias, imp));
                                }
                                else {
                                    const exposedOperators = treeUtils_1.TreeUtils.descendantsOfType(exposingList, "operator_identifier");
                                    exposedOperators.forEach((exposedOperator) => {
                                        const foundNode = exposedFromRemoteModule.get(exposedOperator.text);
                                        if (foundNode) {
                                            this.exposedNodesToImports(foundNode, moduleName, foundModule).forEach((imp) => result.set(imp.alias, imp));
                                        }
                                    });
                                    const exposedValues = treeUtils_1.TreeUtils.findAllNamedChildrenOfType("exposed_value", exposingList);
                                    exposedValues === null || exposedValues === void 0 ? void 0 : exposedValues.forEach((exposedValue) => {
                                        const foundNode = exposedFromRemoteModule.get(exposedValue.text);
                                        if (foundNode) {
                                            this.exposedNodesToImports(foundNode, moduleName, foundModule).forEach((imp) => result.set(imp.alias, imp));
                                        }
                                    });
                                    const exposedTypes = treeUtils_1.TreeUtils.findAllNamedChildrenOfType("exposed_type", exposingList);
                                    exposedTypes === null || exposedTypes === void 0 ? void 0 : exposedTypes.forEach((exposedType) => {
                                        var _a;
                                        const typeName = (_a = treeUtils_1.TreeUtils.findFirstNamedChildOfType("upper_case_identifier", exposedType)) === null || _a === void 0 ? void 0 : _a.text;
                                        const exposedUnionConstructors = !!treeUtils_1.TreeUtils.findFirstNamedChildOfType("exposed_union_constructors", exposedType);
                                        if (typeName) {
                                            const foundNode = exposedFromRemoteModule.get(typeName);
                                            if (foundNode) {
                                                this.exposedNodesToImports(foundNode, moduleName, foundModule, exposedUnionConstructors).forEach((imp) => result.set(imp.alias, imp));
                                            }
                                        }
                                    });
                                }
                            }
                        }
                    }
                }
            }
        });
        exports.importsTime += perf_hooks_1.performance.now() - start;
        return result;
    }
    static getPrefixedImports(moduleName, importNode, exposed, uri, maintainerAndPackageName) {
        const result = [];
        const importedAs = this.findImportAsClause(importNode);
        const importPrefix = importedAs ? importedAs : moduleName;
        exposed.forEach((element, name) => {
            switch (element.type) {
                case "Function":
                case "Port":
                case "TypeAlias":
                    result.push({
                        alias: `${importPrefix}.${name}`,
                        fromModuleName: moduleName,
                        fromUri: uri,
                        maintainerAndPackageName,
                        node: element.syntaxNode,
                        type: element.type,
                        explicitlyExposed: false,
                    });
                    break;
                case "Type":
                    result.push({
                        alias: `${importPrefix}.${name}`,
                        fromModuleName: moduleName,
                        fromUri: uri,
                        maintainerAndPackageName,
                        node: element.syntaxNode,
                        type: element.type,
                        explicitlyExposed: false,
                    });
                    if (element.exposedUnionConstructors) {
                        result.push(...element.exposedUnionConstructors.map((a) => {
                            return {
                                alias: `${importPrefix}.${a.name}`,
                                fromModuleName: moduleName,
                                fromUri: uri,
                                maintainerAndPackageName,
                                node: a.syntaxNode,
                                type: "UnionConstructor",
                                explicitlyExposed: false,
                            };
                        }));
                    }
                    break;
                // Do not handle operators, they are not valid if prefixed
            }
        });
        return result;
    }
    static getVirtualImports() {
        if (this.cachedVirtualImports) {
            return this.cachedVirtualImports;
        }
        const virtualImports = `
  import Basics exposing (..)
  import List exposing (List, (::))
  import Maybe exposing (Maybe(..))
  import Result exposing (Result(..))
  import String exposing (String)
  import Char exposing (Char)
  import Tuple

  import Debug

  import Platform exposing ( Program )
  import Platform.Cmd as Cmd exposing ( Cmd )
  import Platform.Sub as Sub exposing ( Sub )
      `;
        const parser = tsyringe_1.container.resolve("Parser");
        const importTree = parser.parse(virtualImports);
        return (this.cachedVirtualImports = importTree.rootNode.children);
    }
    static findImportAsClause(importNode) {
        const asClause = treeUtils_1.TreeUtils.findFirstNamedChildOfType("as_clause", importNode);
        if (asClause) {
            const newName = treeUtils_1.TreeUtils.findFirstNamedChildOfType("upper_case_identifier", asClause);
            if (newName) {
                return newName.text;
            }
        }
    }
    static getAllExposedCompletions(exposed, moduleName, uri, maintainerAndPackageName) {
        const result = [];
        // These need to be added additionally, as they are nested
        Array.from(exposed.values())
            .filter((it) => it.type === "Type")
            .forEach((element) => {
            if (element.exposedUnionConstructors) {
                result.push(...element.exposedUnionConstructors.map((a) => {
                    return {
                        alias: `${a.name}`,
                        fromModuleName: moduleName,
                        fromUri: uri,
                        maintainerAndPackageName,
                        node: a.syntaxNode,
                        type: "UnionConstructor",
                        explicitlyExposed: false,
                    };
                }));
            }
        });
        return [
            ...result,
            ...Array.from(exposed.values()).map((element) => {
                return {
                    alias: element.name,
                    fromModuleName: moduleName,
                    fromUri: uri,
                    maintainerAndPackageName,
                    node: element.syntaxNode,
                    type: element.type,
                    explicitlyExposed: false,
                };
            }),
        ];
    }
    static exposedNodesToImports(exposedNode, moduleName, foundModule, includeUnionConstructors = false) {
        var _a, _b;
        return [
            {
                alias: exposedNode.name,
                fromModuleName: moduleName,
                fromUri: foundModule.uri,
                maintainerAndPackageName: foundModule.maintainerAndPackageName,
                node: exposedNode.syntaxNode,
                type: exposedNode.type,
                explicitlyExposed: true,
            },
        ].concat(includeUnionConstructors
            ? (_b = (_a = exposedNode.exposedUnionConstructors) === null || _a === void 0 ? void 0 : _a.map((b) => {
                return {
                    alias: b.name,
                    fromModuleName: moduleName,
                    fromUri: foundModule.uri,
                    maintainerAndPackageName: foundModule.maintainerAndPackageName,
                    node: b.syntaxNode,
                    type: "UnionConstructor",
                    explicitlyExposed: false,
                };
            })) !== null && _b !== void 0 ? _b : [] : []);
    }
}
exports.Imports = Imports;
//# sourceMappingURL=imports.js.map