import { CodeAction, Command, Diagnostic as LspDiagnostic, Position, Range, TextEdit } from "vscode-languageserver";
import { ICodeActionParams } from "./paramsExtensions";
export interface ICodeActionRegistration {
    errorCodes: string[];
    fixId: string;
    getCodeActions(params: ICodeActionParams): CodeAction[] | undefined;
    getFixAllCodeAction(params: ICodeActionParams): CodeAction | undefined;
}
export interface IRefactorCodeAction extends CodeAction {
    data: {
        uri: string;
        refactorName: string;
        actionName: string;
        range: Range;
        renamePosition?: Position;
    };
}
export interface IRefactorEdit {
    edits?: TextEdit[];
    renamePosition?: Position;
}
export interface IRefactorRegistration {
    getAvailableActions(params: ICodeActionParams): IRefactorCodeAction[];
    getEditsForAction(params: ICodeActionParams, actionName: string): IRefactorEdit;
}
export declare class CodeActionProvider {
    private connection;
    private settings;
    private elmMake;
    private elmDiagnostics;
    private diagnosticsProvider;
    private static errorCodeToRegistrationMap;
    private static refactorRegistrations;
    constructor();
    static registerCodeAction(registration: ICodeActionRegistration): void;
    static registerRefactorAction(name: string, registration: IRefactorRegistration): void;
    private static getDiagnostics;
    private static forEachDiagnostic;
    static getCodeAction(params: ICodeActionParams, title: string, edits: TextEdit[], command?: Command): CodeAction;
    static getFixAllCodeAction(title: string, params: ICodeActionParams, errorCodes: string[], fixId: string, callback: (edits: TextEdit[], diagnostic: LspDiagnostic) => void): CodeAction;
    protected onCodeAction(params: ICodeActionParams): CodeAction[] | undefined;
    private onCodeActionResolve;
    private addDiagnosticToCodeAction;
}
