import { CancellationToken } from "vscode-languageserver";
import { URI } from "vscode-uri";
import { IElmWorkspace } from "../elmWorkspace";
import { ITreeContainer } from "../forest";
export interface IParams {
    program: IElmWorkspace;
    sourceFile: ITreeContainer;
}
/**
 * Identifies the relevant ElmWorkspace for a given ParamType, either directly
 * (getElmWorkspaceFor) or when an event handler receives a ParamType
 * (handle) it returns a params object with a combined type ParamType and IParams
 * which has the program and sourceFile.
 */
export declare class ElmWorkspaceMatcher<ParamType> {
    private getUriFor;
    private elmWorkspaces;
    constructor(getUriFor: (param: ParamType) => URI);
    handle<ResultType>(handler: (param: ParamType & IParams, token?: CancellationToken) => ResultType): (param: ParamType, token?: CancellationToken) => ResultType;
    handleResolve<ResultType>(handler: (param: ParamType, program: IElmWorkspace, sourceFile: ITreeContainer, token?: CancellationToken) => ResultType): (param: ParamType, token?: CancellationToken) => ResultType;
    /**
     * @deprecated Use handle() instead, which returns a params with the program and source file in it
     */
    handlerForWorkspace<ResultType>(handler: (param: ParamType, program: IElmWorkspace, token?: CancellationToken) => ResultType): (param: ParamType, token?: CancellationToken) => ResultType;
    getProgramFor(param: ParamType): IElmWorkspace;
    getSourceFileFor(param: ParamType, program: IElmWorkspace): ITreeContainer;
}
