"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ElmWorkspaceMatcher = void 0;
const tsyringe_1 = require("tsyringe");
const noWorkspaceContainsError_1 = require("./noWorkspaceContainsError");
/**
 * Identifies the relevant ElmWorkspace for a given ParamType, either directly
 * (getElmWorkspaceFor) or when an event handler receives a ParamType
 * (handle) it returns a params object with a combined type ParamType and IParams
 * which has the program and sourceFile.
 */
class ElmWorkspaceMatcher {
    constructor(getUriFor) {
        this.getUriFor = getUriFor;
        this.elmWorkspaces = tsyringe_1.container.resolve("ElmWorkspaces");
    }
    handle(handler) {
        return (param, token) => {
            const program = this.getProgramFor(param);
            return handler(Object.assign(Object.assign({}, param), { program, sourceFile: this.getSourceFileFor(param, program) }), token);
        };
    }
    handleResolve(handler) {
        return (param, token) => {
            const program = this.getProgramFor(param);
            return handler(param, program, this.getSourceFileFor(param, program), token);
        };
    }
    /**
     * @deprecated Use handle() instead, which returns a params with the program and source file in it
     */
    handlerForWorkspace(handler) {
        return (param, token) => {
            return handler(param, this.getProgramFor(param), token);
        };
    }
    getProgramFor(param) {
        const uri = this.getUriFor(param);
        const workspace = 
        // first look for a workspace where the file has been parsed to a tree
        this.elmWorkspaces.find((ws) => ws.hasDocument(uri)) ||
            // fallback: find a workspace where the file is in the source-directories
            this.elmWorkspaces.find((ws) => ws.isInSourceDirectory(uri.fsPath));
        if (!workspace) {
            throw new noWorkspaceContainsError_1.NoWorkspaceContainsError(this.getUriFor(param));
        }
        return workspace;
    }
    getSourceFileFor(param, program) {
        const uri = this.getUriFor(param).toString();
        return program.getForest().getByUri(uri);
    }
}
exports.ElmWorkspaceMatcher = ElmWorkspaceMatcher;
//# sourceMappingURL=elmWorkspaceMatcher.js.map