"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ImportUtils = void 0;
const ranking_1 = require("../providers/ranking");
const treeUtils_1 = require("./treeUtils");
class ImportUtils {
    static getPossibleImports(program, sourceFile) {
        const currentModule = sourceFile === null || sourceFile === void 0 ? void 0 : sourceFile.moduleName;
        const exposedValues = [];
        // Find all exposed values that could be imported
        program.getImportableModules(sourceFile).forEach(({ uri, moduleName }) => {
            if (uri !== sourceFile.uri && moduleName !== "Basics") {
                const tree = program.getSourceFile(uri);
                if (tree) {
                    exposedValues.push(...ImportUtils.getPossibleImportsOfTree(tree));
                }
            }
        });
        exposedValues.sort((a, b) => {
            if (!a.package && b.package) {
                return -1;
            }
            else if (a.package && !b.package) {
                return 1;
            }
            else if (a.package && b.package) {
                return ranking_1.comparePackageRanking(a.package, b.package);
            }
            else {
                if (!currentModule) {
                    return 0;
                }
                // Sort packages that are in closest to the current module first
                const aScore = this.comparisonScore(currentModule, a.module);
                const bScore = this.comparisonScore(currentModule, b.module);
                if (aScore > bScore) {
                    return -1;
                }
                else if (bScore > aScore) {
                    return 1;
                }
                else {
                    return 0;
                }
            }
        });
        return exposedValues;
    }
    static getPossibleImportsOfTree(tree) {
        var _a;
        const exposedValues = [];
        (_a = tree.exposing) === null || _a === void 0 ? void 0 : _a.forEach((exposed) => {
            var _a;
            const module = tree.moduleName;
            if (module) {
                exposedValues.push({
                    module,
                    value: exposed.name,
                    package: tree.maintainerAndPackageName,
                    type: exposed.type,
                    node: exposed.syntaxNode,
                });
                (_a = exposed.exposedUnionConstructors) === null || _a === void 0 ? void 0 : _a.forEach((exp) => {
                    var _a;
                    if (exp.syntaxNode.parent) {
                        const value = (_a = treeUtils_1.TreeUtils.findFirstNamedChildOfType("upper_case_identifier", exp.syntaxNode.parent)) === null || _a === void 0 ? void 0 : _a.text;
                        if (value) {
                            exposedValues.push({
                                module,
                                value: exp.name,
                                valueToImport: `${value}(..)`,
                                package: tree.maintainerAndPackageName,
                                type: "UnionConstructor",
                                node: exp.syntaxNode,
                            });
                        }
                    }
                });
            }
        });
        return exposedValues;
    }
    static comparisonScore(source, target) {
        let score = 0;
        while (score < Math.min(source.length, target.length) &&
            source.charAt(score) === target.charAt(score)) {
            score++;
        }
        return score;
    }
}
exports.ImportUtils = ImportUtils;
//# sourceMappingURL=importUtils.js.map