"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.References = void 0;
const treeUtils_1 = require("./treeUtils");
const utils_1 = require("./utils");
class References {
    static find(definitionNode, elmWorkspace) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u, _v, _w, _x;
        const references = [];
        const forest = elmWorkspace.getForest();
        const checker = elmWorkspace.getTypeChecker();
        if (definitionNode) {
            const refSourceTree = forest.getByUri(definitionNode.uri);
            if (refSourceTree) {
                const imports = {};
                forest.treeMap.forEach((treeContainer) => {
                    imports[treeContainer.uri] = checker.getAllImports(treeContainer);
                });
                const moduleNameNode = treeUtils_1.TreeUtils.getModuleNameNode(refSourceTree.tree);
                switch (definitionNode.nodeType) {
                    case "Function":
                        {
                            const annotationNameNode = this.getFunctionAnnotationNameNodeFromDefinition(definitionNode.node);
                            if (annotationNameNode && refSourceTree.writeable) {
                                references.push({
                                    node: annotationNameNode,
                                    uri: definitionNode.uri,
                                });
                            }
                            const functionNameNode = treeUtils_1.TreeUtils.getFunctionNameNodeFromDefinition(definitionNode.node);
                            if (functionNameNode) {
                                const functionName = functionNameNode.text;
                                if (refSourceTree.writeable) {
                                    references.push({
                                        node: functionNameNode,
                                        uri: definitionNode.uri,
                                    });
                                }
                                const localFunctions = ((_a = definitionNode.node.parent) === null || _a === void 0 ? void 0 : _a.parent) &&
                                    ((_b = definitionNode.node.parent) === null || _b === void 0 ? void 0 : _b.parent.type) === "let_in_expr" && ((_c = definitionNode.node.parent) === null || _c === void 0 ? void 0 : _c.parent.lastNamedChild)
                                    ? this.findFunctionCalls(definitionNode.node.parent.parent.lastNamedChild, functionName)
                                    : this.findFunctionCalls(refSourceTree.tree.rootNode, functionName);
                                if (localFunctions && refSourceTree.writeable) {
                                    references.push(...localFunctions.map((node) => {
                                        return { node, uri: definitionNode.uri };
                                    }));
                                }
                                const isExposedFunction = treeUtils_1.TreeUtils.isExposedFunctionOrPort(refSourceTree.tree, functionName);
                                if (isExposedFunction) {
                                    const moduleDeclarationNode = treeUtils_1.TreeUtils.findModuleDeclaration(refSourceTree.tree);
                                    if (moduleDeclarationNode) {
                                        const exposedNode = treeUtils_1.TreeUtils.findExposedFunctionNode(moduleDeclarationNode, functionName);
                                        if (exposedNode && refSourceTree.writeable) {
                                            references.push({
                                                node: exposedNode,
                                                uri: definitionNode.uri,
                                            });
                                        }
                                    }
                                    if (isExposedFunction && moduleNameNode) {
                                        const moduleName = moduleNameNode.text;
                                        for (const uri in imports) {
                                            if (uri === definitionNode.uri) {
                                                continue;
                                            }
                                            const otherTreeContainer = forest.getByUri(uri);
                                            if (!otherTreeContainer) {
                                                continue;
                                            }
                                            const importedModuleAlias = (_d = treeUtils_1.TreeUtils.findImportAliasOfModule(moduleName, otherTreeContainer.tree)) !== null && _d !== void 0 ? _d : moduleName;
                                            const element = imports[uri];
                                            const filter = (imp) => imp.type === "Function" &&
                                                imp.fromModuleName === moduleName;
                                            // Find the function in the other module's imports
                                            const found = (_e = element.get(functionName, filter)) !== null && _e !== void 0 ? _e : element.get(`${importedModuleAlias}.${functionName}`, filter);
                                            if (found) {
                                                if (otherTreeContainer &&
                                                    otherTreeContainer.writeable) {
                                                    const importClause = (_g = (_f = otherTreeContainer.symbolLinks) === null || _f === void 0 ? void 0 : _f.get(otherTreeContainer.tree.rootNode)) === null || _g === void 0 ? void 0 : _g.get(importedModuleAlias);
                                                    // Add node from exposing list
                                                    if ((importClause === null || importClause === void 0 ? void 0 : importClause.type) === "Import") {
                                                        const exposedNode = treeUtils_1.TreeUtils.findExposedFunctionNode(importClause.node, functionName);
                                                        if (exposedNode) {
                                                            references.push({
                                                                node: exposedNode,
                                                                uri,
                                                            });
                                                        }
                                                    }
                                                    // Find all function calls in the other tree
                                                    const functions = this.findFunctionCalls(otherTreeContainer.tree.rootNode, found === null || found === void 0 ? void 0 : found.alias);
                                                    if (functions) {
                                                        references.push(...functions.map((node) => {
                                                            return { node, uri };
                                                        }));
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        break;
                    case "Port":
                        {
                            const portNameNode = treeUtils_1.TreeUtils.getTypeOrTypeAliasOrPortNameNodeFromDefinition(definitionNode.node);
                            if (portNameNode) {
                                const portName = portNameNode.text;
                                if (refSourceTree.writeable) {
                                    references.push({
                                        node: portNameNode,
                                        uri: definitionNode.uri,
                                    });
                                }
                                const localCallsToPort = this.findFunctionCalls(refSourceTree.tree.rootNode, portName);
                                if (localCallsToPort && refSourceTree.writeable) {
                                    references.push(...localCallsToPort.map((node) => {
                                        return { node, uri: definitionNode.uri };
                                    }));
                                }
                                const isExposedPort = treeUtils_1.TreeUtils.isExposedFunctionOrPort(refSourceTree.tree, portName);
                                if (isExposedPort) {
                                    const moduleDeclarationNode = treeUtils_1.TreeUtils.findModuleDeclaration(refSourceTree.tree);
                                    if (moduleDeclarationNode) {
                                        const exposedNode = treeUtils_1.TreeUtils.findExposedFunctionNode(moduleDeclarationNode, portName);
                                        if (exposedNode && refSourceTree.writeable) {
                                            references.push({
                                                node: exposedNode,
                                                uri: definitionNode.uri,
                                            });
                                        }
                                    }
                                    if (isExposedPort && moduleNameNode) {
                                        const moduleName = moduleNameNode.text;
                                        for (const uri in imports) {
                                            if (uri === definitionNode.uri) {
                                                continue;
                                            }
                                            const otherTreeContainer = forest.getByUri(uri);
                                            if (!otherTreeContainer) {
                                                continue;
                                            }
                                            const importedModuleAlias = (_h = treeUtils_1.TreeUtils.findImportAliasOfModule(moduleName, otherTreeContainer.tree)) !== null && _h !== void 0 ? _h : moduleName;
                                            const element = imports[uri];
                                            const filter = (imp) => imp.type === "Port" &&
                                                imp.fromModuleName === moduleName;
                                            // Find the function in the other module's imports
                                            const found = (_j = element.get(portName, filter)) !== null && _j !== void 0 ? _j : element.get(`${importedModuleAlias}.${portName}`, filter);
                                            if (found) {
                                                if (otherTreeContainer &&
                                                    otherTreeContainer.writeable) {
                                                    const importClause = (_l = (_k = otherTreeContainer.symbolLinks) === null || _k === void 0 ? void 0 : _k.get(otherTreeContainer.tree.rootNode)) === null || _l === void 0 ? void 0 : _l.get(importedModuleAlias);
                                                    // Add node from exposing list
                                                    if ((importClause === null || importClause === void 0 ? void 0 : importClause.type) === "Import") {
                                                        const exposedNode = treeUtils_1.TreeUtils.findExposedFunctionNode(importClause.node, portName);
                                                        if (exposedNode) {
                                                            references.push({
                                                                node: exposedNode,
                                                                uri,
                                                            });
                                                        }
                                                    }
                                                    // Find all function calls in the other tree
                                                    const functions = this.findFunctionCalls(otherTreeContainer.tree.rootNode, found === null || found === void 0 ? void 0 : found.alias);
                                                    if (functions) {
                                                        references.push(...functions.map((node) => {
                                                            return { node, uri };
                                                        }));
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        break;
                    case "Type":
                    case "TypeAlias":
                        {
                            const typeOrTypeAliasNameNode = treeUtils_1.TreeUtils.getTypeOrTypeAliasOrPortNameNodeFromDefinition(definitionNode.node);
                            if (typeOrTypeAliasNameNode) {
                                const typeOrTypeAliasName = typeOrTypeAliasNameNode.text;
                                if (refSourceTree.writeable) {
                                    references.push({
                                        node: typeOrTypeAliasNameNode,
                                        uri: definitionNode.uri,
                                    });
                                }
                                const localFunctions = treeUtils_1.TreeUtils.findTypeOrTypeAliasCalls(refSourceTree.tree, typeOrTypeAliasName);
                                if (localFunctions && refSourceTree.writeable) {
                                    references.push(...localFunctions.map((node) => {
                                        return { node, uri: definitionNode.uri };
                                    }));
                                }
                                const isExposed = treeUtils_1.TreeUtils.isExposedTypeOrTypeAlias(refSourceTree.tree, typeOrTypeAliasName);
                                if (isExposed) {
                                    const moduleDeclarationNode = treeUtils_1.TreeUtils.findModuleDeclaration(refSourceTree.tree);
                                    if (moduleDeclarationNode) {
                                        const exposedNode = treeUtils_1.TreeUtils.findExposedTypeOrTypeAliasNode(moduleDeclarationNode, typeOrTypeAliasName);
                                        if (exposedNode && refSourceTree.writeable) {
                                            references.push({
                                                node: exposedNode,
                                                uri: definitionNode.uri,
                                            });
                                        }
                                    }
                                    if (isExposed && moduleNameNode) {
                                        const moduleName = moduleNameNode.text;
                                        for (const uri in imports) {
                                            if (uri === definitionNode.uri) {
                                                continue;
                                            }
                                            const otherTreeContainer = forest.getByUri(uri);
                                            if (!otherTreeContainer) {
                                                continue;
                                            }
                                            const importedModuleAlias = (_m = treeUtils_1.TreeUtils.findImportAliasOfModule(moduleName, otherTreeContainer.tree)) !== null && _m !== void 0 ? _m : moduleName;
                                            const element = imports[uri];
                                            const filter = (imp) => (imp.type === "Type" || imp.type === "TypeAlias") &&
                                                imp.fromModuleName === moduleName;
                                            // Find the type or type alias in the other module's imports
                                            const found = (_o = element.get(typeOrTypeAliasName, filter)) !== null && _o !== void 0 ? _o : element.get(`${importedModuleAlias}.${typeOrTypeAliasName}`, filter);
                                            if (found) {
                                                if (otherTreeContainer &&
                                                    otherTreeContainer.writeable) {
                                                    const importClause = (_q = (_p = otherTreeContainer.symbolLinks) === null || _p === void 0 ? void 0 : _p.get(otherTreeContainer.tree.rootNode)) === null || _q === void 0 ? void 0 : _q.get(importedModuleAlias);
                                                    if ((importClause === null || importClause === void 0 ? void 0 : importClause.type) === "Import") {
                                                        const exposedNode = treeUtils_1.TreeUtils.findExposedTypeOrTypeAliasNode(importClause.node, typeOrTypeAliasNameNode.text);
                                                        if (exposedNode) {
                                                            references.push({
                                                                node: exposedNode,
                                                                uri,
                                                            });
                                                        }
                                                    }
                                                    const typeOrTypeAliasCalls = treeUtils_1.TreeUtils.findTypeOrTypeAliasCalls(otherTreeContainer.tree, found.alias);
                                                    if (typeOrTypeAliasCalls) {
                                                        references.push(...typeOrTypeAliasCalls.map((node) => {
                                                            return { node, uri };
                                                        }));
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        break;
                    case "Module":
                        if (moduleNameNode) {
                            if (refSourceTree.writeable) {
                                references.push({
                                    node: moduleNameNode,
                                    uri: definitionNode.uri,
                                });
                            }
                            for (const uri in imports) {
                                if (uri === definitionNode.uri) {
                                    continue;
                                }
                                const sourceFileToCheck = forest.getByUri(uri);
                                if (!sourceFileToCheck || !sourceFileToCheck.writeable) {
                                    continue;
                                }
                                const moduleNameToLookFor = (_r = treeUtils_1.TreeUtils.findImportAliasOfModule(moduleNameNode.text, sourceFileToCheck.tree)) !== null && _r !== void 0 ? _r : moduleNameNode.text;
                                // Check if it is imported
                                const imported = (_t = (_s = sourceFileToCheck.symbolLinks) === null || _s === void 0 ? void 0 : _s.get(sourceFileToCheck.tree.rootNode)) === null || _t === void 0 ? void 0 : _t.get(moduleNameToLookFor, (symbol) => symbol.type === "Import");
                                if (imported) {
                                    const importNameNode = checker.findImportModuleNameNode(moduleNameToLookFor, sourceFileToCheck);
                                    if (importNameNode) {
                                        references.push({ node: importNameNode, uri });
                                    }
                                }
                                // If it is not imported as an alias, find all references in file
                                if (imported && moduleNameToLookFor === moduleNameNode.text) {
                                    sourceFileToCheck.tree.rootNode
                                        .descendantsOfType("value_expr")
                                        .forEach((valueNode) => {
                                        if (RegExp(`${moduleNameToLookFor}.[a-z].*`).exec(valueNode.text)) {
                                            references.push({ node: valueNode, uri });
                                        }
                                    });
                                }
                            }
                        }
                        break;
                    case "FunctionParameter":
                        if (refSourceTree.writeable) {
                            references.push({
                                node: definitionNode.node,
                                uri: definitionNode.uri,
                            });
                            const valueDeclaration = treeUtils_1.TreeUtils.findParentOfType("function_declaration_left", definitionNode.node);
                            if (valueDeclaration &&
                                valueDeclaration.nextNamedSibling &&
                                valueDeclaration.nextNamedSibling.nextNamedSibling) {
                                const functionBody = valueDeclaration.nextNamedSibling.nextNamedSibling;
                                if (functionBody) {
                                    const parameters = this.findParameterUsage(functionBody, definitionNode.node.text);
                                    if (parameters) {
                                        references.push(...parameters.map((node) => {
                                            return { node, uri: definitionNode.uri };
                                        }));
                                    }
                                }
                            }
                        }
                        break;
                    case "CasePattern":
                        if (refSourceTree.writeable) {
                            references.push({
                                node: definitionNode.node,
                                uri: definitionNode.uri,
                            });
                            if (definitionNode.node.parent &&
                                definitionNode.node.parent.parent &&
                                definitionNode.node.parent.parent.parent &&
                                definitionNode.node.parent.parent.parent.lastNamedChild) {
                                const caseBody = definitionNode.node.parent.parent.parent.lastNamedChild;
                                if (caseBody) {
                                    const parameters = this.findParameterUsage(caseBody, definitionNode.node.text);
                                    if (parameters) {
                                        references.push(...parameters.map((node) => {
                                            return { node, uri: definitionNode.uri };
                                        }));
                                    }
                                }
                            }
                        }
                        break;
                    case "AnonymousFunctionParameter":
                        if (refSourceTree.writeable) {
                            references.push({
                                node: definitionNode.node,
                                uri: definitionNode.uri,
                            });
                            if (definitionNode.node.parent &&
                                definitionNode.node.parent.parent) {
                                const anonymousFunction = definitionNode.node.parent.parent; // TODO this is due to tree sitter matching wrong
                                if (anonymousFunction) {
                                    const parameters = this.findParameterUsage(anonymousFunction, definitionNode.node.text);
                                    if (parameters) {
                                        references.push(...parameters.map((node) => {
                                            return { node, uri: definitionNode.uri };
                                        }));
                                    }
                                }
                            }
                        }
                        break;
                    case "UnionConstructor":
                        if (definitionNode.node.firstChild && moduleNameNode) {
                            const nameNode = definitionNode.node.firstChild;
                            if (refSourceTree.writeable) {
                                references.push({
                                    node: nameNode,
                                    uri: definitionNode.uri,
                                });
                                const unionConstructorCalls = treeUtils_1.TreeUtils.findUnionConstructorCalls(refSourceTree.tree, nameNode.text);
                                if (unionConstructorCalls) {
                                    references.push(...unionConstructorCalls.map((a) => {
                                        return { node: a, uri: definitionNode.uri };
                                    }));
                                }
                            }
                            for (const uri in imports) {
                                if (uri === definitionNode.uri) {
                                    continue;
                                }
                                const element = imports[uri];
                                const found = (_u = element.get(nameNode.text)) !== null && _u !== void 0 ? _u : element.get(`${moduleNameNode.text}.${nameNode.text}`);
                                const needsToBeChecked = (found === null || found === void 0 ? void 0 : found.fromModuleName) === moduleNameNode.text &&
                                    found.type === "UnionConstructor";
                                if (needsToBeChecked) {
                                    const treeToCheck = forest.getByUri(uri);
                                    if (treeToCheck && treeToCheck.writeable) {
                                        const unionConstructorCallsFromOtherFiles = treeUtils_1.TreeUtils.findUnionConstructorCalls(treeToCheck.tree, nameNode.text);
                                        if (unionConstructorCallsFromOtherFiles) {
                                            references.push(...unionConstructorCallsFromOtherFiles.map((node) => {
                                                return { node, uri };
                                            }));
                                        }
                                    }
                                }
                            }
                        }
                        break;
                    case "FieldType":
                        {
                            const fieldName = definitionNode.node.childForFieldName("name");
                            if (fieldName) {
                                references.push({
                                    node: fieldName,
                                    uri: definitionNode.uri,
                                });
                                references.push(...this.getFieldReferences(fieldName.text, definitionNode, refSourceTree, elmWorkspace));
                                for (const uri in imports) {
                                    if (uri === definitionNode.uri) {
                                        continue;
                                    }
                                    const needsToBeChecked = (_w = (_v = forest
                                        .getByUri(uri)) === null || _v === void 0 ? void 0 : _v.resolvedModules) === null || _w === void 0 ? void 0 : _w.get((_x = moduleNameNode === null || moduleNameNode === void 0 ? void 0 : moduleNameNode.text) !== null && _x !== void 0 ? _x : "");
                                    if (needsToBeChecked) {
                                        const treeToCheck = forest.getByUri(uri);
                                        if (treeToCheck && treeToCheck.writeable) {
                                            references.push(...this.getFieldReferences(fieldName.text, definitionNode, treeToCheck, elmWorkspace));
                                        }
                                    }
                                }
                            }
                        }
                        break;
                    default:
                        break;
                }
            }
        }
        return references;
    }
    static findOperator(node, elmWorkspace) {
        var _a, _b, _c;
        const functionNameNode = treeUtils_1.TreeUtils.getFunctionNameNodeFromDefinition(node);
        if (functionNameNode) {
            const infixRef = (_c = (_b = (_a = elmWorkspace
                .getForest()
                .getByUri(node.tree.uri)) === null || _a === void 0 ? void 0 : _a.symbolLinks) === null || _b === void 0 ? void 0 : _b.get(node.tree.rootNode)) === null || _c === void 0 ? void 0 : _c.get(functionNameNode.text, (s) => s.node.type === "infix_declaration");
            return infixRef === null || infixRef === void 0 ? void 0 : infixRef.node;
        }
    }
    static findFunctionCalls(node, functionName) {
        const functions = this.findAllFunctionCallsAndParameters(node);
        const result = functions
            .filter((a) => a.text === functionName)
            .map((a) => a.lastChild);
        return result.length === 0 ? undefined : result;
    }
    static findAllFunctionCallsAndParameters(node) {
        let functions = treeUtils_1.TreeUtils.descendantsOfType(node, "value_expr");
        if (functions.length > 0) {
            functions = functions
                .filter((a) => a.firstChild && a.firstChild.type === "value_qid")
                .map((a) => a.firstChild);
        }
        return functions;
    }
    static findParameterUsage(node, functionName) {
        const parameters = [
            ...this.findAllFunctionCallsAndParameters(node),
            ...this.findAllRecordBaseIdentifiers(node),
        ];
        const result = parameters.filter((a) => a.text === functionName);
        return result.length === 0 ? undefined : result;
    }
    static findAllRecordBaseIdentifiers(node) {
        return treeUtils_1.TreeUtils.descendantsOfType(node, "record_base_identifier");
    }
    static getFunctionAnnotationNameNodeFromDefinition(node) {
        if (node.parent &&
            node.parent.previousNamedSibling &&
            node.parent.previousNamedSibling.type === "type_annotation" &&
            node.parent.previousNamedSibling.firstChild &&
            node.parent.previousNamedSibling.firstChild.type ===
                "lower_case_identifier") {
            return node.parent.previousNamedSibling.firstChild;
        }
    }
    static findFieldUsages(tree, fieldName) {
        return tree.rootNode
            .descendantsOfType([
            "field",
            "field_accessor_function_expr",
            "field_access_expr",
            "record_pattern",
        ])
            .map((field) => {
            var _a;
            if (field.type === "record_pattern") {
                const lowerPattern = field.namedChildren.find((pattern) => pattern.type === "lower_pattern" && pattern.text === fieldName);
                if (lowerPattern) {
                    const declaration = treeUtils_1.TreeUtils.findParentOfType("value_declaration", lowerPattern);
                    const patternRefs = (_a = declaration === null || declaration === void 0 ? void 0 : declaration.descendantsOfType("value_qid").filter((ref) => ref.text === fieldName)) !== null && _a !== void 0 ? _a : [];
                    return [lowerPattern, ...patternRefs];
                }
            }
            return [field];
        })
            .reduce((a, b) => a.concat(b), [])
            .map((field) => treeUtils_1.TreeUtils.findFirstNamedChildOfType("lower_case_identifier", field))
            .filter(utils_1.Utils.notUndefinedOrNull.bind(this))
            .filter((field) => field.text === fieldName);
    }
    static getFieldReferences(fieldName, definition, treeContainer, elmWorkspace) {
        const references = [];
        const fieldUsages = References.findFieldUsages(treeContainer.tree, fieldName);
        fieldUsages.forEach((field) => {
            const fieldDef = elmWorkspace
                .getTypeChecker()
                .findDefinition(field, treeContainer);
            if ((fieldDef === null || fieldDef === void 0 ? void 0 : fieldDef.node.id) === definition.node.id) {
                references.push({
                    node: field,
                    uri: treeContainer.uri,
                });
            }
        });
        return references;
    }
}
exports.References = References;
//# sourceMappingURL=references.js.map