"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SymbolInformationTranslator = void 0;
const vscode_languageserver_1 = require("vscode-languageserver");
class SymbolInformationTranslator {
    static translateNodeToSymbolInformation(uri, node) {
        var _a, _b;
        switch (node.type) {
            case "value_declaration":
                {
                    const functionName = (_b = (_a = node.firstChild) === null || _a === void 0 ? void 0 : _a.firstChild) === null || _b === void 0 ? void 0 : _b.text;
                    if (functionName) {
                        return this.createSymbolInformation(functionName, node, vscode_languageserver_1.SymbolKind.Function, uri);
                    }
                }
                break;
            case "module_declaration": {
                const nameNodeModule = node.childForFieldName("name");
                if (nameNodeModule) {
                    return this.createSymbolInformation(nameNodeModule.text, node, vscode_languageserver_1.SymbolKind.Module, uri);
                }
                else {
                    return;
                }
            }
            case "type_declaration": {
                const nameNodeTypeDec = node.childForFieldName("name");
                if (nameNodeTypeDec) {
                    return this.createSymbolInformation(nameNodeTypeDec.text, node, vscode_languageserver_1.SymbolKind.Enum, uri);
                }
                else {
                    return;
                }
            }
            case "type_alias_declaration": {
                const nameNodeAliasDec = node.childForFieldName("name");
                if (nameNodeAliasDec) {
                    return this.createSymbolInformation(nameNodeAliasDec.text, node, vscode_languageserver_1.SymbolKind.Struct, uri);
                }
                else {
                    return;
                }
            }
            case "union_variant":
                return this.createSymbolInformation(node.text, node, vscode_languageserver_1.SymbolKind.EnumMember, uri);
            default:
                break;
        }
    }
    static createSymbolInformation(name, node, symbolKind, uri) {
        return vscode_languageserver_1.SymbolInformation.create(name, symbolKind, vscode_languageserver_1.Range.create(vscode_languageserver_1.Position.create(node.startPosition.row, node.startPosition.column), vscode_languageserver_1.Position.create(node.endPosition.row, node.endPosition.column)), uri);
    }
}
exports.SymbolInformationTranslator = SymbolInformationTranslator;
//# sourceMappingURL=symbolTranslator.js.map