import { DiagnosticSeverity, Range } from "vscode-languageserver";
import { SyntaxNode } from "web-tree-sitter";
import { DiagnosticSource } from "../../providers/diagnostics/diagnosticSource";
export interface Diagnostic {
    code: string;
    message: string;
    source: DiagnosticSource;
    severity: DiagnosticSeverity;
    range: Range;
    uri: string;
}
export declare function errorWithEndNode(node: SyntaxNode, diagnostic: IDiagnosticMessage, endNode?: SyntaxNode, ...args: (string | number)[]): Diagnostic;
export declare function error(node: SyntaxNode, diagnostic: IDiagnosticMessage, ...args: (string | number)[]): Diagnostic;
export interface IDiagnosticMessage {
    code: string;
    message: string;
    severity: DiagnosticSeverity;
}
export declare const Diagnostics: {
    ArgumentCount: IDiagnosticMessage;
    CyclicDefinition: IDiagnosticMessage;
    FieldAccessOnNonRecord: IDiagnosticMessage;
    General: IDiagnosticMessage;
    ImportMissing: IDiagnosticMessage;
    InvalidPattern: IDiagnosticMessage;
    MissingTypeAnnotation: IDiagnosticMessage;
    MissingValue: IDiagnosticMessage;
    NonAssociativeOperator: IDiagnosticMessage;
    ParameterCountError: IDiagnosticMessage;
    Parsing: IDiagnosticMessage;
    PartialPattern: IDiagnosticMessage;
    RecordBaseId: IDiagnosticMessage;
    RecordField: IDiagnosticMessage;
    RecursiveAlias: (n: number) => IDiagnosticMessage;
    RecursiveDeclaration: (n: number) => IDiagnosticMessage;
    RecursiveLet: (n: number) => IDiagnosticMessage;
    Redefinition: IDiagnosticMessage;
    TypeArgumentCount: IDiagnosticMessage;
    TypeMismatch: IDiagnosticMessage;
};
