"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Diagnostics = exports.error = exports.errorWithEndNode = void 0;
/* eslint-disable @typescript-eslint/naming-convention */
const vscode_languageserver_1 = require("vscode-languageserver");
const positionUtil_1 = require("../../positionUtil");
function format(text, ...args) {
    return text.replace(/{(\d+)}/g, (_match, index) => `${args[+index]}`);
}
function errorWithEndNode(node, diagnostic, endNode, ...args) {
    var _a;
    return {
        range: {
            start: positionUtil_1.PositionUtil.FROM_TS_POSITION(node.startPosition).toVSPosition(),
            end: positionUtil_1.PositionUtil.FROM_TS_POSITION((_a = endNode === null || endNode === void 0 ? void 0 : endNode.endPosition) !== null && _a !== void 0 ? _a : node.endPosition).toVSPosition(),
        },
        message: format(diagnostic.message, ...args),
        code: diagnostic.code,
        severity: diagnostic.severity,
        source: "Elm",
        uri: node.tree.uri,
    };
}
exports.errorWithEndNode = errorWithEndNode;
function error(node, diagnostic, ...args) {
    return errorWithEndNode(node, diagnostic, undefined, ...args);
}
exports.error = error;
function diag(code, message, severity) {
    return { code, message, severity };
}
exports.Diagnostics = {
    ArgumentCount: diag("argument_count", "`{0}` is not a function, but it was given {1} arguments.", vscode_languageserver_1.DiagnosticSeverity.Error),
    CyclicDefinition: diag("cyclic_definition", "Value cannot be defined in terms of itself.", vscode_languageserver_1.DiagnosticSeverity.Error),
    FieldAccessOnNonRecord: diag("field_access_on_non_record", "Cannot access fields on non-record type: `{0}`.", vscode_languageserver_1.DiagnosticSeverity.Error),
    General: diag("general", "General error: {0}", vscode_languageserver_1.DiagnosticSeverity.Error),
    ImportMissing: diag("import_resolve", "Could not find a module to import named `{0}` in `dependencies` or `source-directories`.", vscode_languageserver_1.DiagnosticSeverity.Error),
    InvalidPattern: diag("invalid_pattern", "Invalid pattern error.\nExpected: `{0}`\nFound: `{1}`", vscode_languageserver_1.DiagnosticSeverity.Error),
    MissingTypeAnnotation: diag("missing_type_annotation", "Missing type annotation: `{0}`.", vscode_languageserver_1.DiagnosticSeverity.Information),
    MissingValue: diag("missing_value", "No definition found for `{0}`.", vscode_languageserver_1.DiagnosticSeverity.Error),
    NonAssociativeOperator: diag("non_associative_operator", "Non associative operator.", vscode_languageserver_1.DiagnosticSeverity.Error),
    ParameterCountError: diag("parameter_count", "The {0} `{1}` expects {2} arguments, but got {3} instead.", vscode_languageserver_1.DiagnosticSeverity.Error),
    Parsing: diag("parsing", "Parsing error.", vscode_languageserver_1.DiagnosticSeverity.Error),
    PartialPattern: diag("partial_pattern", "Pattern does not cover all possibilities.", vscode_languageserver_1.DiagnosticSeverity.Error),
    RecordBaseId: diag("record_base_id", "Type must be a record, instead found: `{0}`.", vscode_languageserver_1.DiagnosticSeverity.Error),
    RecordField: diag("record_field", "The record does not have a `{0}` field.", vscode_languageserver_1.DiagnosticSeverity.Error),
    RecursiveAlias: (n) => diag("recursive_alias", n < 2
        ? "Alias problem. This type alias is recursive, forming an infinite type."
        : `Alias problem. This type alias is part of a mutually recursive set of type aliases:\n${Array.from(Array(n).keys())
            .map((i) => `{${i}}`)
            .join(" -> ")}`, vscode_languageserver_1.DiagnosticSeverity.Error),
    RecursiveDeclaration: (n) => diag("recursive_declaration", n < 2
        ? "Cyclic definition. The value `{0}` is defined directly in terms of itself, causing an infinite loop."
        : `Cyclic definition. The value \`{0}\`  depends on itself through the following chain of definitions:\n${Array.from(Array(n).keys())
            .map((i) => `{${i}}`)
            .join(" -> ")}`, vscode_languageserver_1.DiagnosticSeverity.Error),
    RecursiveLet: (n) => diag("recursive_let", n < 2
        ? "Cyclic value. The value `{0}` is defined directly in terms of itself, causing an infinite loop."
        : `Cyclic value. The value \`{0}\` depends on itself through the following chain of definitions:\n${Array.from(Array(n).keys())
            .map((i) => `{${i}}`)
            .join(" -> ")}`, vscode_languageserver_1.DiagnosticSeverity.Error),
    Redefinition: diag("redefinition", "A value named `{0}` is already defined.", vscode_languageserver_1.DiagnosticSeverity.Error),
    TypeArgumentCount: diag("type_argument_count", "The type expected {0} arguments, but got {1} instead.", vscode_languageserver_1.DiagnosticSeverity.Error),
    TypeMismatch: diag("type_mismatch", "Type mismatch error.\nExpected: `{0}`\nFound: `{1}`", vscode_languageserver_1.DiagnosticSeverity.Error),
};
//# sourceMappingURL=diagnostics.js.map