"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SyntaxNodeMap = void 0;
class SyntaxNodeMap {
    constructor() {
        this.map = new Map();
    }
    set(key, value) {
        this.map.set(key.id, value);
    }
    get(key) {
        return this.map.get(key.id);
    }
    getOrSet(key, setter) {
        const existing = this.get(key);
        if (existing) {
            return existing;
        }
        const result = setter();
        this.set(key, result);
        return result;
    }
    has(key) {
        return this.map.has(key.id);
    }
    mapValues(callback) {
        this.map.forEach((val, key) => {
            this.map.set(key, callback(val));
        });
    }
    forEach(callback) {
        this.map.forEach((val, key) => callback(val, { id: key }));
    }
    clear() {
        this.map.clear();
    }
    delete(key) {
        this.map.delete(key.id);
    }
}
exports.SyntaxNodeMap = SyntaxNodeMap;
//# sourceMappingURL=syntaxNodeMap.js.map