import { SyntaxNode } from "web-tree-sitter";
import { NodeType } from "../treeUtils";
import { IElmWorkspace } from "../../elmWorkspace";
import { Type } from "./typeInference";
import { ITreeContainer } from "../../forest";
import { Imports } from "../../imports";
import { ICancellationToken } from "../../cancellation";
import { Diagnostic } from "./diagnostics";
export declare let bindTime: number;
export declare function resetBindTime(): void;
export interface DefinitionResult {
    node: SyntaxNode;
    uri: string;
    nodeType: NodeType;
}
export interface TypeChecker {
    findType: (node: SyntaxNode) => Type;
    findDefinition: (node: SyntaxNode, treeContainer: ITreeContainer) => DefinitionResult | undefined;
    findDefinitionShallow: (node: SyntaxNode, treeContainer: ITreeContainer) => DefinitionResult | undefined;
    getAllImports: (treeContainer: ITreeContainer) => Imports;
    getQualifierForName: (treeContainer: ITreeContainer, module: string, name: string) => string | undefined;
    typeToString: (t: Type, treeContainer?: ITreeContainer) => string;
    getDiagnostics: (treeContainer: ITreeContainer, cancellationToken?: ICancellationToken) => Diagnostic[];
    getDiagnosticsAsync: (treeContainer: ITreeContainer, token?: ICancellationToken, cancelCallback?: () => boolean) => Promise<Diagnostic[]>;
    getSuggestionDiagnostics: (treeContainer: ITreeContainer, cancellationToken?: ICancellationToken) => Diagnostic[];
    findImportModuleNameNode: (moduleNameOrAlias: string, treeContainer: ITreeContainer) => SyntaxNode | undefined;
}
export declare function createTypeChecker(workspace: IElmWorkspace): TypeChecker;
